/* memchurn:  Quick-and-dirty memory tester

   Copyright (C) 2004-2007 by Brian Lindholm.
   This file is part of the littleutils utility set.

   The memchurn utility is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   The memchurn utility is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   the littleutils.  If not, see <http://www.gnu.org/licenses/>. */


#include "../config.h"
#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#define OPTEND -1
#else
#define OPTEND EOF
#endif

#ifdef __MINGW32__
extern int getopt (int argc, char * const *argv, const char *optstring);
extern char *optarg;
extern int optind;
#endif

#ifdef HAVE_DMALLOC_H
#include <dmalloc.h>
#endif


/* help function */

static void
help (FILE *where)
{
  fprintf (where,
    "memchurn " PACKAGE_VERSION "\n"
    "usage: memchurn [-m megs] [-v(erbose)]\n");
}


/* main program */

int
main (int argc, char *argv[])
{
  int i, j, ints_per_meg, megs, opt, verbose;
  int **meg_list;

  /* parse options */

  verbose = 0;
  megs = 64;
  while ((opt = getopt (argc, argv, "hm:v")) != OPTEND)
    switch (opt)
      {
      case 'h':
        help (stdout);
        return (0);
      case 'm':
        megs = atoi (optarg);
        break;
      case 'v':
        verbose = 1;
        break;
      case '?':
        help (stderr);
        return (1);
      }

  /* finalize options */

  if (megs < 1)
    megs = 1;
  else if (megs > 4095)
    megs = 4095;
  ints_per_meg = 1024 * 1024 / ((int) sizeof (int));

  /* create pointer list */

  meg_list = (int **) malloc (megs * sizeof (int *));
  if (meg_list == NULL)
    {
      fprintf (stderr, "memchurn error: premature out of memory\n");
      return (1);
    }
  for (i = 0; i < megs; i++)
    meg_list[i] = (int *) NULL;

  /* write the memory */

  for (i = 0; i < megs; i++)
    {
      meg_list[i] = (int *) malloc (ints_per_meg * sizeof (int));
      if (meg_list[i] == NULL)
        {
          fprintf (stderr, "memchurn warning: out of memory at meg %d\n", i);
          megs = i;
        }
      else
        {
          for (j = 0; j < ints_per_meg; j++)
            meg_list[i][j] = j;
          if (verbose)
            fprintf (stdout, "memchurn message: writing meg %d\n", i);
        }
    }
  fprintf (stdout, "memchurn success: %d megs written\n", megs);


  /* check the memory */

  for (i = 0; i < megs; i++)
    {
      for (j = 0; j < ints_per_meg; j++)
        if (meg_list[i][j] != j)
          {
            fprintf (stderr, "memchurn error: failed memory check at meg %d\n", i);
            return (1);
          }
      if (verbose)
        fprintf (stdout, "memchurn message: checking meg %d\n", i);
     }
  fprintf (stdout, "memchurn success: %d megs scanned\n", megs);

  /* free the memory */

  for (i = 0; i < megs; i++)
    free (meg_list[i]);
  free (meg_list);
  fprintf (stdout, "memchurn success: %d megs freed\n", megs);

  /* exit */

  return (0);
}
